@@ -23,13 +23,8 @@ def get_lockdata():
     else:
         start_len = "qq"
 
-    if sys.platform in ('netbsd1', 'netbsd2', 'netbsd3',
-                        'Darwin1.2', 'darwin',
-                        'freebsd2', 'freebsd3', 'freebsd4', 'freebsd5',
-                        'freebsd6', 'freebsd7', 'freebsd8',
-                        'bsdos2', 'bsdos3', 'bsdos4',
-                        'openbsd', 'openbsd2', 'openbsd3', 'openbsd4',
-                        'openbsd5'):
+    if (sys.platform.startswith(('netbsd', 'freebsd', 'openbsd', 'bsdos'))
+        or sys.platform == 'darwin'):
         if struct.calcsize('l') == 8:
             off_t = 'l'
             pid_t = 'i'
