@@ -1,7 +1,5 @@
 # Adapted from test_file.py by Daniel Stutzbach
 
-from __future__ import unicode_literals
-
 import sys
 import os
 import errno
@@ -9,11 +7,8 @@ import unittest
 from array import array
 from weakref import proxy
 from functools import wraps
-from UserList import UserList
 
-from test.test_support import TESTFN, check_warnings, run_unittest, make_bad_fd
-from test.test_support import py3k_bytes as bytes, cpython_only, check_py3k_warnings
-from test.script_helper import run_python
+from test.support import TESTFN, check_warnings, run_unittest, make_bad_fd
 
 from _io import FileIO as _FileIO
 
@@ -65,32 +60,12 @@ class AutoFileTests(unittest.TestCase):
 
     def testReadinto(self):
         # verify readinto
-        self.f.write(b"\x01\x02")
+        self.f.write(bytes([1, 2]))
         self.f.close()
-        a = array(b'b', b'x'*10)
+        a = array('b', b'x'*10)
         self.f = _FileIO(TESTFN, 'r')
         n = self.f.readinto(a)
-        self.assertEqual(array(b'b', [1, 2]), a[:n])
-
-    def testWritelinesList(self):
-        l = [b'123', b'456']
-        self.f.writelines(l)
-        self.f.close()
-        self.f = _FileIO(TESTFN, 'rb')
-        buf = self.f.read()
-        self.assertEqual(buf, b'123456')
-
-    def testWritelinesUserList(self):
-        l = UserList([b'123', b'456'])
-        self.f.writelines(l)
-        self.f.close()
-        self.f = _FileIO(TESTFN, 'rb')
-        buf = self.f.read()
-        self.assertEqual(buf, b'123456')
-
-    def testWritelinesError(self):
-        self.assertRaises(TypeError, self.f.writelines, [1, 2, 3])
-        self.assertRaises(TypeError, self.f.writelines, None)
+        self.assertEqual(array('b', [1, 2]), a[:n])
 
     def test_none_args(self):
         self.f.write(b"hi\nbye\nabc")
@@ -101,39 +76,36 @@ class AutoFileTests(unittest.TestCase):
         self.assertEqual(self.f.readline(None), b"hi\n")
         self.assertEqual(self.f.readlines(None), [b"bye\n", b"abc"])
 
-    def testWriteUnicode(self):
-        with check_py3k_warnings():
-            self.f.write(u'')
+    def test_reject(self):
+        self.assertRaises(TypeError, self.f.write, "Hello!")
 
     def testRepr(self):
-        self.assertEqual(repr(self.f), "<_io.FileIO name=%r mode='%s'>"
-                                       % (self.f.name, self.f.mode))
+        self.assertEqual(repr(self.f), "<_io.FileIO name=%r mode=%r>"
+                                        % (self.f.name, self.f.mode))
         del self.f.name
-        self.assertEqual(repr(self.f), "<_io.FileIO fd=%r mode='%s'>"
-                                       % (self.f.fileno(), self.f.mode))
+        self.assertEqual(repr(self.f), "<_io.FileIO fd=%r mode=%r>"
+                                        % (self.f.fileno(), self.f.mode))
         self.f.close()
         self.assertEqual(repr(self.f), "<_io.FileIO [closed]>")
 
     def testErrors(self):
         f = self.f
-        self.assertFalse(f.isatty())
-        self.assertFalse(f.closed)
+        self.assertTrue(not f.isatty())
+        self.assertTrue(not f.closed)
         #self.assertEqual(f.name, TESTFN)
         self.assertRaises(ValueError, f.read, 10) # Open for reading
         f.close()
         self.assertTrue(f.closed)
         f = _FileIO(TESTFN, 'r')
         self.assertRaises(TypeError, f.readinto, "")
-        self.assertFalse(f.closed)
+        self.assertTrue(not f.closed)
         f.close()
         self.assertTrue(f.closed)
 
     def testMethods(self):
-        methods = ['fileno', 'isatty', 'seekable', 'readable', 'writable',
-                   'read', 'readall', 'readline', 'readlines',
-                   'tell', 'truncate', 'flush']
-        if sys.platform.startswith('atheos'):
-            methods.remove('truncate')
+        methods = ['fileno', 'isatty', 'read', 'readinto',
+                   'seek', 'tell', 'truncate', 'write', 'seekable',
+                   'readable', 'writable']
 
         self.f.close()
         self.assertTrue(self.f.closed)
@@ -143,15 +115,6 @@ class AutoFileTests(unittest.TestCase):
             # should raise on closed file
             self.assertRaises(ValueError, method)
 
-        self.assertRaises(ValueError, self.f.readinto) # XXX should be TypeError?
-        self.assertRaises(ValueError, self.f.readinto, bytearray(1))
-        self.assertRaises(ValueError, self.f.seek)
-        self.assertRaises(ValueError, self.f.seek, 0)
-        self.assertRaises(ValueError, self.f.write)
-        self.assertRaises(ValueError, self.f.write, b'')
-        self.assertRaises(TypeError, self.f.writelines)
-        self.assertRaises(ValueError, self.f.writelines, b'')
-
     def testOpendir(self):
         # Issue 3703: opening a directory should fill the errno
         # Windows always returns "[Errno 13]: Permission denied
@@ -164,14 +127,6 @@ class AutoFileTests(unittest.TestCase):
         else:
             self.fail("Should have raised IOError")
 
-    @unittest.skipIf(os.name == 'nt', "test only works on a POSIX-like system")
-    def testOpenDirFD(self):
-        fd = os.open('.', os.O_RDONLY)
-        with self.assertRaises(IOError) as cm:
-            _FileIO(fd, 'r')
-        os.close(fd)
-        self.assertEqual(cm.exception.errno, errno.EISDIR)
-
     #A set of functions testing that we get expected behaviour if someone has
     #manually closed the internal file descriptor.  First, a decorator:
     def ClosedFD(func):
@@ -270,7 +225,7 @@ class AutoFileTests(unittest.TestCase):
     @ClosedFDRaises
     def testErrnoOnClosedReadinto(self, f):
         f = self.ReopenForRead()
-        a = array(b'b', b'x'*10)
+        a = array('b', b'x'*10)
         f.readinto(a)
 
 class OtherFileTests(unittest.TestCase):
@@ -295,30 +250,29 @@ class OtherFileTests(unittest.TestCase):
             self.assertEqual(f.seekable(), True)
             self.assertEqual(f.isatty(), False)
             f.close()
+
+            if sys.platform != "win32":
+                try:
+                    f = _FileIO("/dev/tty", "a")
+                except EnvironmentError:
+                    # When run in a cron job there just aren't any
+                    # ttys, so skip the test.  This also handles other
+                    # OS'es that don't support /dev/tty.
+                    pass
+                else:
+                    self.assertEqual(f.readable(), False)
+                    self.assertEqual(f.writable(), True)
+                    if sys.platform != "darwin" and \
+                       'bsd' not in sys.platform and \
+                       not sys.platform.startswith('sunos'):
+                        # Somehow /dev/tty appears seekable on some BSDs
+                        self.assertEqual(f.seekable(), False)
+                    self.assertEqual(f.isatty(), True)
+                    f.close()
         finally:
             os.unlink(TESTFN)
 
-    @unittest.skipIf(sys.platform == 'win32', 'no ttys on Windows')
-    def testAblesOnTTY(self):
-        try:
-            f = _FileIO("/dev/tty", "a")
-        except EnvironmentError:
-            # When run in a cron job there just aren't any
-            # ttys, so skip the test.  This also handles other
-            # OS'es that don't support /dev/tty.
-            self.skipTest('need /dev/tty')
-        else:
-            self.assertEqual(f.readable(), False)
-            self.assertEqual(f.writable(), True)
-            if sys.platform != "darwin" and \
-               'bsd' not in sys.platform and \
-               not sys.platform.startswith(('sunos', 'aix')):
-                # Somehow /dev/tty appears seekable on some BSDs
-                self.assertEqual(f.seekable(), False)
-            self.assertEqual(f.isatty(), True)
-            f.close()
-
-    def testInvalidModeStrings(self):
+    def testModeStrings(self):
         # check invalid mode strings
         for mode in ("", "aU", "wU+", "rw", "rt"):
             try:
@@ -329,21 +283,6 @@ class OtherFileTests(unittest.TestCase):
                 f.close()
                 self.fail('%r is an invalid file mode' % mode)
 
-    def testModeStrings(self):
-        # test that the mode attribute is correct for various mode strings
-        # given as init args
-        try:
-            for modes in [('w', 'wb'), ('wb', 'wb'), ('wb+', 'rb+'),
-                          ('w+b', 'rb+'), ('a', 'ab'), ('ab', 'ab'),
-                          ('ab+', 'ab+'), ('a+b', 'ab+'), ('r', 'rb'),
-                          ('rb', 'rb'), ('rb+', 'rb+'), ('r+b', 'rb+')]:
-                # read modes are last so that TESTFN will exist first
-                with _FileIO(TESTFN, modes[0]) as f:
-                    self.assertEqual(f.mode, modes[1])
-        finally:
-            if os.path.exists(TESTFN):
-                os.unlink(TESTFN)
-
     def testUnicodeOpen(self):
         # verify repr works for unicode too
         f = _FileIO(str(TESTFN), "w")
@@ -355,7 +294,8 @@ class OtherFileTests(unittest.TestCase):
         try:
             fn = TESTFN.encode("ascii")
         except UnicodeEncodeError:
-            self.skipTest('could not encode %r to ascii' % TESTFN)
+            # Skip test
+            return
         f = _FileIO(fn, "w")
         try:
             f.write(b"abc")
@@ -368,7 +308,7 @@ class OtherFileTests(unittest.TestCase):
     def testConstructorHandlesNULChars(self):
         fn_with_NUL = 'foo\0bar'
         self.assertRaises(TypeError, _FileIO, fn_with_NUL, 'w')
-        self.assertRaises(TypeError, _FileIO, fn_with_NUL.encode('ascii'), 'w')
+        self.assertRaises(TypeError, _FileIO, bytes(fn_with_NUL, 'ascii'), 'w')
 
     def testInvalidFd(self):
         self.assertRaises(ValueError, _FileIO, -10)
@@ -377,13 +317,6 @@ class OtherFileTests(unittest.TestCase):
             import msvcrt
             self.assertRaises(IOError, msvcrt.get_osfhandle, make_bad_fd())
 
-    @cpython_only
-    def testInvalidFd_overflow(self):
-        # Issue 15989
-        import _testcapi
-        self.assertRaises(TypeError, _FileIO, _testcapi.INT_MAX + 1)
-        self.assertRaises(TypeError, _FileIO, _testcapi.INT_MIN - 1)
-
     def testBadModeArgument(self):
         # verify that we get a sensible error message for bad mode argument
         bad_mode = "qwerty"
@@ -470,37 +403,6 @@ class OtherFileTests(unittest.TestCase):
             self.assertRaises(ValueError, _FileIO, "/some/invalid/name", "rt")
             self.assertEqual(w.warnings, [])
 
-    def test_surrogates(self):
-        # Issue #8438: try to open a filename containing surrogates.
-        # It should either fail because the file doesn't exist or the filename
-        # can't be represented using the filesystem encoding, but not because
-        # of a LookupError for the error handler "surrogateescape".
-        filename = u'\udc80.txt'
-        try:
-            with _FileIO(filename):
-                pass
-        except (UnicodeEncodeError, IOError):
-            pass
-        # Spawn a separate Python process with a different "file system
-        # default encoding", to exercise this further.
-        env = dict(os.environ)
-        env[b'LC_CTYPE'] = b'C'
-        _, out = run_python('-c', 'import _io; _io.FileIO(%r)' % filename, env=env)
-        if ('UnicodeEncodeError' not in out and not
-                ( ('IOError: [Errno 2] No such file or directory' in out) or
-                  ('IOError: [Errno 22] Invalid argument' in out) ) ):
-            self.fail('Bad output: %r' % out)
-
-    def testUnclosedFDOnException(self):
-        class MyException(Exception): pass
-        class MyFileIO(_FileIO):
-            def __setattr__(self, name, value):
-                if name == "name":
-                    raise MyException("blocked setting name")
-                return super(MyFileIO, self).__setattr__(name, value)
-        fd = os.open(__file__, os.O_RDONLY)
-        self.assertRaises(MyException, MyFileIO, fd)
-        os.close(fd)  # should not raise OSError(EBADF)
 
 def test_main():
     # Historically, these tests have been sloppy about removing TESTFN.
