@@ -1,18 +1,16 @@
 # Python test set -- part 1, grammar.
 # This just tests whether the parser accepts them all.
 
-from test.test_support import run_unittest, check_syntax_error, \
-                              check_py3k_warnings
+from test.support import run_unittest, check_syntax_error
 import unittest
 import sys
-import warnings
 # testing import *
 from sys import *
 
 
 class TokenTests(unittest.TestCase):
 
-    def test_backslash(self):
+    def testBackslash(self):
         # Backslash means line continuation:
         x = 1 \
         + 1
@@ -22,40 +20,33 @@ class TokenTests(unittest.TestCase):
         x = 0
         self.assertEqual(x, 0, 'backslash ending comment')
 
-    def test_plain_integers(self):
+    def testPlainIntegers(self):
         self.assertEqual(type(000), type(0))
         self.assertEqual(0xff, 255)
-        self.assertEqual(0377, 255)
         self.assertEqual(0o377, 255)
-        self.assertEqual(2147483647, 017777777777)
         self.assertEqual(2147483647, 0o17777777777)
         self.assertEqual(0b1001, 9)
         # "0x" is not a valid literal
         self.assertRaises(SyntaxError, eval, "0x")
-        from sys import maxint
-        if maxint == 2147483647:
+        from sys import maxsize
+        if maxsize == 2147483647:
             self.assertEqual(-2147483647-1, -0o20000000000)
             # XXX -2147483648
-            self.assertTrue(037777777777 > 0)
             self.assertTrue(0o37777777777 > 0)
             self.assertTrue(0xffffffff > 0)
             self.assertTrue(0b1111111111111111111111111111111 > 0)
-            for s in ('2147483648', '040000000000', '0o40000000000',
-                      '0x100000000',
+            for s in ('2147483648', '0o40000000000', '0x100000000',
                       '0b10000000000000000000000000000000'):
                 try:
                     x = eval(s)
                 except OverflowError:
                     self.fail("OverflowError on huge integer literal %r" % s)
-        elif maxint == 9223372036854775807:
-            self.assertEqual(-9223372036854775807-1, -01000000000000000000000)
+        elif maxsize == 9223372036854775807:
             self.assertEqual(-9223372036854775807-1, -0o1000000000000000000000)
-            self.assertTrue(01777777777777777777777 > 0)
             self.assertTrue(0o1777777777777777777777 > 0)
             self.assertTrue(0xffffffffffffffff > 0)
             self.assertTrue(0b11111111111111111111111111111111111111111111111111111111111111 > 0)
-            for s in '9223372036854775808', '02000000000000000000000', \
-                     '0o2000000000000000000000', \
+            for s in '9223372036854775808', '0o2000000000000000000000', \
                      '0x10000000000000000', \
                      '0b100000000000000000000000000000000000000000000000000000000000000':
                 try:
@@ -63,19 +54,19 @@ class TokenTests(unittest.TestCase):
                 except OverflowError:
                     self.fail("OverflowError on huge integer literal %r" % s)
         else:
-            self.fail('Weird maxint value %r' % maxint)
+            self.fail('Weird maxsize value %r' % maxsize)
 
-    def test_long_integers(self):
-        x = 0L
-        x = 0l
-        x = 0xffffffffffffffffL
-        x = 0xffffffffffffffffl
-        x = 077777777777777777L
-        x = 077777777777777777l
-        x = 123456789012345678901234567890L
-        x = 123456789012345678901234567890l
+    def testLongIntegers(self):
+        x = 0
+        x = 0xffffffffffffffff
+        x = 0Xffffffffffffffff
+        x = 0o77777777777777777
+        x = 0O77777777777777777
+        x = 123456789012345678901234567890
+        x = 0b100000000000000000000000000000000000000000000000000000000000000000000
+        x = 0B111111111111111111111111111111111111111111111111111111111111111111111
 
-    def test_floats(self):
+    def testFloats(self):
         x = 3.14
         x = 314.
         x = 0.314
@@ -89,13 +80,7 @@ class TokenTests(unittest.TestCase):
         x = .3e14
         x = 3.1e4
 
-    def test_float_exponent_tokenization(self):
-        # See issue 21642.
-        self.assertEqual(1 if 1else 0, 1)
-        self.assertEqual(1 if 0else 0, 0)
-        self.assertRaises(SyntaxError, eval, "0 if 1Else 0")
-
-    def test_string_literals(self):
+    def testStringLiterals(self):
         x = ''; y = ""; self.assertTrue(len(x) == 0 and x == y)
         x = '\''; y = "'"; self.assertTrue(len(x) == 1 and x == y and ord(x) == 39)
         x = '"'; y = "\""; self.assertTrue(len(x) == 1 and x == y and ord(x) == 34)
@@ -135,6 +120,10 @@ the \'lazy\' dog.\n\
 '
         self.assertEqual(x, y)
 
+    def testEllipsis(self):
+        x = ...
+        self.assertTrue(x is Ellipsis)
+        self.assertRaises(SyntaxError, eval, ".. .")
 
 class GrammarTests(unittest.TestCase):
 
@@ -147,58 +136,42 @@ class GrammarTests(unittest.TestCase):
     # expr_input: testlist NEWLINE
     # XXX Hard to test -- used only in calls to input()
 
-    def test_eval_input(self):
+    def testEvalInput(self):
         # testlist ENDMARKER
         x = eval('1, 0 or 1')
 
-    def test_funcdef(self):
-        ### 'def' NAME parameters ':' suite
-        ### parameters: '(' [varargslist] ')'
-        ### varargslist: (fpdef ['=' test] ',')* ('*' NAME [',' ('**'|'*' '*') NAME]
-        ###            | ('**'|'*' '*') NAME)
-        ###            | fpdef ['=' test] (',' fpdef ['=' test])* [',']
-        ### fpdef: NAME | '(' fplist ')'
-        ### fplist: fpdef (',' fpdef)* [',']
-        ### arglist: (argument ',')* (argument | *' test [',' '**' test] | '**' test)
-        ### argument: [test '='] test   # Really [keyword '='] test
+    def testFuncdef(self):
+        ### [decorators] 'def' NAME parameters ['->' test] ':' suite
+        ### decorator: '@' dotted_name [ '(' [arglist] ')' ] NEWLINE
+        ### decorators: decorator+
+        ### parameters: '(' [typedargslist] ')'
+        ### typedargslist: ((tfpdef ['=' test] ',')*
+        ###                ('*' [tfpdef] (',' tfpdef ['=' test])* [',' '**' tfpdef] | '**' tfpdef)
+        ###                | tfpdef ['=' test] (',' tfpdef ['=' test])* [','])
+        ### tfpdef: NAME [':' test]
+        ### varargslist: ((vfpdef ['=' test] ',')*
+        ###              ('*' [vfpdef] (',' vfpdef ['=' test])*  [',' '**' vfpdef] | '**' vfpdef)
+        ###              | vfpdef ['=' test] (',' vfpdef ['=' test])* [','])
+        ### vfpdef: NAME
         def f1(): pass
         f1()
         f1(*())
         f1(*(), **{})
         def f2(one_argument): pass
         def f3(two, arguments): pass
-        with check_py3k_warnings(('tuple parameter unpacking has been removed',
-                                  SyntaxWarning)):
-            exec('def f4(two, (compound, (argument, list))): pass')
-            exec('def f5((compound, first), two): pass')
-        self.assertEqual(f2.func_code.co_varnames, ('one_argument',))
-        self.assertEqual(f3.func_code.co_varnames, ('two', 'arguments'))
-        if sys.platform.startswith('java'):
-            self.assertEqual(f4.func_code.co_varnames,
-                   ('two', '(compound, (argument, list))', 'compound', 'argument',
-                                'list',))
-            self.assertEqual(f5.func_code.co_varnames,
-                   ('(compound, first)', 'two', 'compound', 'first'))
-        else:
-            self.assertEqual(f4.func_code.co_varnames,
-                  ('two', '.1', 'compound', 'argument',  'list'))
-            self.assertEqual(f5.func_code.co_varnames,
-                  ('.0', 'two', 'compound', 'first'))
+        self.assertEqual(f2.__code__.co_varnames, ('one_argument',))
+        self.assertEqual(f3.__code__.co_varnames, ('two', 'arguments'))
         def a1(one_arg,): pass
         def a2(two, args,): pass
         def v0(*rest): pass
         def v1(a, *rest): pass
         def v2(a, b, *rest): pass
-        with check_py3k_warnings(('tuple parameter unpacking has been removed',
-                                  SyntaxWarning)):
-            exec('def v3(a, (b, c), *rest): return a, b, c, rest')
 
         f1()
         f2(1)
         f2(1,)
         f3(1, 2)
         f3(1, 2,)
-        f4(1, (2, (3, 4)))
         v0()
         v0(1)
         v0(1,)
@@ -213,24 +186,12 @@ class GrammarTests(unittest.TestCase):
         v2(1,2,3)
         v2(1,2,3,4)
         v2(1,2,3,4,5,6,7,8,9,0)
-        v3(1,(2,3))
-        v3(1,(2,3),4)
-        v3(1,(2,3),4,5,6,7,8,9,0)
 
-        # ceval unpacks the formal arguments into the first argcount names;
-        # thus, the names nested inside tuples must appear after these names.
-        if sys.platform.startswith('java'):
-            self.assertEqual(v3.func_code.co_varnames, ('a', '(b, c)', 'rest', 'b', 'c'))
-        else:
-            self.assertEqual(v3.func_code.co_varnames, ('a', '.1', 'rest', 'b', 'c'))
-        self.assertEqual(v3(1, (2, 3), 4), (1, 2, 3, (4,)))
         def d01(a=1): pass
         d01()
         d01(1)
         d01(*(1,))
-        d01(*[] or [2])
         d01(**{'a':2})
-        d01(**{'a':2} or {})
         def d11(a, b=1): pass
         d11(1)
         d11(1, 2)
@@ -298,14 +259,24 @@ class GrammarTests(unittest.TestCase):
         d22v(*(1, 2, 3, 4))
         d22v(1, 2, *(3, 4, 5))
         d22v(1, *(2, 3), **{'d': 4})
-        with check_py3k_warnings(('parenthesized argument names are invalid',
-                                  SyntaxWarning)):
-            exec('def d31v((x)): pass')
-        with check_py3k_warnings(('tuple parameter unpacking has been removed',
-                                  SyntaxWarning)):
-            exec('def d32v((x,)): pass')
-        d31v(1)
-        d32v((1,))
+
+        # keyword argument type tests
+        try:
+            str('x', **{b'foo':1 })
+        except TypeError:
+            pass
+        else:
+            self.fail('Bytes should not work as keyword argument names')
+        # keyword only argument tests
+        def pos0key1(*, key): return key
+        pos0key1(key=100)
+        def pos2key2(p1, p2, *, k1, k2=100): return p1,p2,k1,k2
+        pos2key2(1, 2, k1=100)
+        pos2key2(1, 2, k1=100, k2=200)
+        pos2key2(1, 2, k2=100, k1=200)
+        def pos2key2dict(p1, p2, *, k1=100, k2, **kwarg): return p1,p2,k1,k2,kwarg
+        pos2key2dict(1,2,k2=100,tokwarg1=100,tokwarg2=200)
+        pos2key2dict(1,2,tokwarg1=100,tokwarg2=200, k2=100)
 
         # keyword arguments after *arglist
         def f(*args, **kwargs):
@@ -314,24 +285,51 @@ class GrammarTests(unittest.TestCase):
                                                     {'x':2, 'y':5}))
         self.assertRaises(SyntaxError, eval, "f(1, *(2,3), 4)")
         self.assertRaises(SyntaxError, eval, "f(1, x=2, *(3,4), x=5)")
-        self.assertEqual(f(**{'eggs':'scrambled', 'spam':'fried'}),
-                         ((), {'eggs':'scrambled', 'spam':'fried'}))
-        self.assertEqual(f(spam='fried', **{'eggs':'scrambled'}),
-                         ((), {'eggs':'scrambled', 'spam':'fried'}))
+
+        # argument annotation tests
+        def f(x) -> list: pass
+        self.assertEqual(f.__annotations__, {'return': list})
+        def f(x:int): pass
+        self.assertEqual(f.__annotations__, {'x': int})
+        def f(*x:str): pass
+        self.assertEqual(f.__annotations__, {'x': str})
+        def f(**x:float): pass
+        self.assertEqual(f.__annotations__, {'x': float})
+        def f(x, y:1+2): pass
+        self.assertEqual(f.__annotations__, {'y': 3})
+        def f(a, b:1, c:2, d): pass
+        self.assertEqual(f.__annotations__, {'b': 1, 'c': 2})
+        def f(a, b:1, c:2, d, e:3=4, f=5, *g:6): pass
+        self.assertEqual(f.__annotations__,
+                          {'b': 1, 'c': 2, 'e': 3, 'g': 6})
+        def f(a, b:1, c:2, d, e:3=4, f=5, *g:6, h:7, i=8, j:9=10,
+              **k:11) -> 12: pass
+        self.assertEqual(f.__annotations__,
+                          {'b': 1, 'c': 2, 'e': 3, 'g': 6, 'h': 7, 'j': 9,
+                           'k': 11, 'return': 12})
+        # Check for SF Bug #1697248 - mixing decorators and a return annotation
+        def null(x): return x
+        @null
+        def f(x) -> list: pass
+        self.assertEqual(f.__annotations__, {'return': list})
+
+        # test MAKE_CLOSURE with a variety of oparg's
+        closure = 1
+        def f(): return closure
+        def f(x=1): return closure
+        def f(*, k=1): return closure
+        def f() -> int: return closure
 
         # Check ast errors in *args and *kwargs
         check_syntax_error(self, "f(*g(1=2))")
         check_syntax_error(self, "f(**g(1=2))")
 
-        # Check trailing commas are permitted in funcdef argument list
-        def f(a,): pass
-
-    def test_lambdef(self):
+    def testLambdef(self):
         ### lambdef: 'lambda' [varargslist] ':' test
         l1 = lambda : 0
         self.assertEqual(l1(), 0)
         l2 = lambda : a[d] # XXX just testing the expression
-        l3 = lambda : [2 < x for x in [-1, 3, 0L]]
+        l3 = lambda : [2 < x for x in [-1, 3, 0]]
         self.assertEqual(l3(), [0, 1, 0])
         l4 = lambda x = lambda y = lambda z=1 : z : y() : x()
         self.assertEqual(l4(), 1)
@@ -339,18 +337,16 @@ class GrammarTests(unittest.TestCase):
         self.assertEqual(l5(1, 2), 5)
         self.assertEqual(l5(1, 2, 3), 6)
         check_syntax_error(self, "lambda x: x = 2")
-        with check_py3k_warnings(('tuple parameter unpacking has been removed',
-                                  SyntaxWarning)):
-            check_syntax_error(self, "lambda (None,): None")
-
-        # check that trailing commas are permitted
-        l10 = lambda a,: 0
+        check_syntax_error(self, "lambda (None,): None")
+        l6 = lambda x, y, *, k=20: x+y+k
+        self.assertEqual(l6(1,2), 1+2+20)
+        self.assertEqual(l6(1,2,k=10), 1+2+10)
 
 
     ### stmt: simple_stmt | compound_stmt
     # Tested below
 
-    def test_simple_stmt(self):
+    def testSimpleStmt(self):
         ### simple_stmt: small_stmt (';' small_stmt)* [';']
         x = 1; pass; del x
         def foo():
@@ -358,10 +354,10 @@ class GrammarTests(unittest.TestCase):
             x = 1; pass; del x;
         foo()
 
-    ### small_stmt: expr_stmt | print_stmt  | pass_stmt | del_stmt | flow_stmt | import_stmt | global_stmt | access_stmt | exec_stmt
+    ### small_stmt: expr_stmt | pass_stmt | del_stmt | flow_stmt | import_stmt | global_stmt | access_stmt
     # Tested below
 
-    def test_expr_stmt(self):
+    def testExprStmt(self):
         # (exprlist '=')* exprlist
         1
         1, 2, 3
@@ -374,77 +370,7 @@ class GrammarTests(unittest.TestCase):
         check_syntax_error(self, "x + 1 = 1")
         check_syntax_error(self, "a + 1 = b + 2")
 
-    def test_print_stmt(self):
-        # 'print' (test ',')* [test]
-        import StringIO
-
-        # Can't test printing to real stdout without comparing output
-        # which is not available in unittest.
-        save_stdout = sys.stdout
-        sys.stdout = StringIO.StringIO()
-
-        print 1, 2, 3
-        print 1, 2, 3,
-        print
-        print 0 or 1, 0 or 1,
-        print 0 or 1
-
-        # 'print' '>>' test ','
-        print >> sys.stdout, 1, 2, 3
-        print >> sys.stdout, 1, 2, 3,
-        print >> sys.stdout
-        print >> sys.stdout, 0 or 1, 0 or 1,
-        print >> sys.stdout, 0 or 1
-
-        # test printing to an instance
-        class Gulp:
-            def write(self, msg): pass
-
-        gulp = Gulp()
-        print >> gulp, 1, 2, 3
-        print >> gulp, 1, 2, 3,
-        print >> gulp
-        print >> gulp, 0 or 1, 0 or 1,
-        print >> gulp, 0 or 1
-
-        # test print >> None
-        def driver():
-            oldstdout = sys.stdout
-            sys.stdout = Gulp()
-            try:
-                tellme(Gulp())
-                tellme()
-            finally:
-                sys.stdout = oldstdout
-
-        # we should see this once
-        def tellme(file=sys.stdout):
-            print >> file, 'hello world'
-
-        driver()
-
-        # we should not see this at all
-        def tellme(file=None):
-            print >> file, 'goodbye universe'
-
-        driver()
-
-        self.assertEqual(sys.stdout.getvalue(), '''\
-1 2 3
-1 2 3
-1 1 1
-1 2 3
-1 2 3
-1 1 1
-hello world
-''')
-        sys.stdout = save_stdout
-
-        # syntax errors
-        check_syntax_error(self, 'print ,')
-        check_syntax_error(self, 'print >> x,')
-
-    def test_del_stmt(self):
+    def testDelStmt(self):
         # 'del' exprlist
         abc = [1,2,3]
         x, y, z = abc
@@ -453,18 +379,18 @@ hello world
         del abc
         del x, y, (z, xyz)
 
-    def test_pass_stmt(self):
+    def testPassStmt(self):
         # 'pass'
         pass
 
     # flow_stmt: break_stmt | continue_stmt | return_stmt | raise_stmt
     # Tested below
 
-    def test_break_stmt(self):
+    def testBreakStmt(self):
         # 'break'
         while 1: break
 
-    def test_continue_stmt(self):
+    def testContinueStmt(self):
         # 'continue'
         i = 1
         while i: i = 0; continue
@@ -516,7 +442,7 @@ hello world
                 self.fail("continue then break in try/except in loop broken!")
         test_inner()
 
-    def test_return(self):
+    def testReturn(self):
         # 'return' [testlist]
         def g1(): return
         def g2(): return 1
@@ -524,150 +450,17 @@ hello world
         x = g2()
         check_syntax_error(self, "class foo:return 1")
 
-    def test_break_in_finally(self):
-        count = 0
-        while count < 2:
-            count += 1
-            try:
-                pass
-            finally:
-                break
-        self.assertEqual(count, 1)
-
-        count = 0
-        while count < 2:
-            count += 1
-            try:
-                continue
-            finally:
-                break
-        self.assertEqual(count, 1)
-
-        count = 0
-        while count < 2:
-            count += 1
-            try:
-                1.0/0.0
-            finally:
-                break
-        self.assertEqual(count, 1)
-
-        for count in [0, 1]:
-            self.assertEqual(count, 0)
-            try:
-                pass
-            finally:
-                break
-        self.assertEqual(count, 0)
-
-        for count in [0, 1]:
-            self.assertEqual(count, 0)
-            try:
-                continue
-            finally:
-                break
-        self.assertEqual(count, 0)
-
-        for count in [0, 1]:
-            self.assertEqual(count, 0)
-            try:
-                1.0/0.0
-            finally:
-                break
-        self.assertEqual(count, 0)
-
-    def test_return_in_finally(self):
-        def g1():
-            try:
-                pass
-            finally:
-                return 1
-        self.assertEqual(g1(), 1)
-
-        def g2():
-            try:
-                return 2
-            finally:
-                return 3
-        self.assertEqual(g2(), 3)
-
-        def g3():
-            try:
-                1.0/0.0
-            finally:
-                return 4
-        self.assertEqual(g3(), 4)
-
-    def test_yield(self):
-        # Allowed as standalone statement
-        def g(): yield 1
-        # Allowed as RHS of assignment
-        def g(): x = yield 1
-        # Ordinary yield accepts implicit tuples
-        def g(): yield 1, 1
-        def g(): x = yield 1, 1
-        # Requires parentheses as subexpression
-        def g(): 1, (yield 1)
-        check_syntax_error(self, "def g(): 1, yield 1")
-        # Requires parentheses as call argument
-        def g(): f((yield 1))
-        def g(): f((yield 1), 1)
-        check_syntax_error(self, "def g(): f(yield 1)")
-        check_syntax_error(self, "def g(): f(yield 1, 1)")
-        # Not allowed at top level
-        check_syntax_error(self, "yield")
-        # Not allowed at class scope
+    def testYield(self):
         check_syntax_error(self, "class foo:yield 1")
-        # Check annotation refleak on SyntaxError
-        check_syntax_error(self, "def g(a:(yield)): pass")
 
-    def test_yield_in_comprehensions(self):
-        # Check yield in comprehensions
-        def g(): [x for x in [(yield 1)]]
-
-        def check(code, warntext):
-            with check_py3k_warnings((warntext, DeprecationWarning)):
-                compile(code, '<test string>', 'exec')
-            if sys.py3kwarning:
-                with warnings.catch_warnings():
-                    warnings.filterwarnings('error', category=DeprecationWarning)
-                    with self.assertRaises(SyntaxError) as cm:
-                        compile(code, '<test string>', 'exec')
-                    self.assertIn(warntext, str(cm.exception))
-
-        check("def g(): [(yield x) for x in ()]",
-              "'yield' inside list comprehension")
-        check("def g(): [x for x in () if not (yield x)]",
-              "'yield' inside list comprehension")
-        check("def g(): [y for x in () for y in [(yield x)]]",
-              "'yield' inside list comprehension")
-        check("def g(): {(yield x) for x in ()}",
-              "'yield' inside set comprehension")
-        check("def g(): {(yield x): x for x in ()}",
-              "'yield' inside dict comprehension")
-        check("def g(): {x: (yield x) for x in ()}",
-              "'yield' inside dict comprehension")
-        check("def g(): ((yield x) for x in ())",
-              "'yield' inside generator expression")
-        with check_py3k_warnings(("'yield' inside list comprehension",
-                                  DeprecationWarning)):
-            check_syntax_error(self, "class C: [(yield x) for x in ()]")
-        check("class C: ((yield x) for x in ())",
-              "'yield' inside generator expression")
-        with check_py3k_warnings(("'yield' inside list comprehension",
-                                  DeprecationWarning)):
-            check_syntax_error(self, "[(yield x) for x in ()]")
-        check("((yield x) for x in ())",
-              "'yield' inside generator expression")
-
-    def test_raise(self):
+    def testRaise(self):
         # 'raise' test [',' test]
-        try: raise RuntimeError, 'just testing'
+        try: raise RuntimeError('just testing')
         except RuntimeError: pass
         try: raise KeyboardInterrupt
         except KeyboardInterrupt: pass
 
-    def test_import(self):
+    def testImport(self):
         # 'import' dotted_as_names
         import sys
         import time, sys
@@ -680,45 +473,21 @@ hello world
         from sys import (path, argv)
         from sys import (path, argv,)
 
-    def test_global(self):
+    def testGlobal(self):
         # 'global' NAME (',' NAME)*
         global a
         global a, b
         global one, two, three, four, five, six, seven, eight, nine, ten
 
-    def test_exec(self):
-        # 'exec' expr ['in' expr [',' expr]]
-        z = None
-        del z
-        exec 'z=1+1\n'
-        if z != 2: self.fail('exec \'z=1+1\'\\n')
-        del z
-        exec 'z=1+1'
-        if z != 2: self.fail('exec \'z=1+1\'')
-        z = None
-        del z
-        import types
-        if hasattr(types, "UnicodeType"):
-            exec r"""if 1:
-            exec u'z=1+1\n'
-            if z != 2: self.fail('exec u\'z=1+1\'\\n')
-            del z
-            exec u'z=1+1'
-            if z != 2: self.fail('exec u\'z=1+1\'')"""
-        g = {}
-        exec 'z = 1' in g
-        if '__builtins__' in g: del g['__builtins__']
-        if g != {'z': 1}: self.fail('exec \'z = 1\' in g')
-        g = {}
-        l = {}
+    def testNonlocal(self):
+        # 'nonlocal' NAME (',' NAME)*
+        x = 0
+        y = 0
+        def f():
+            nonlocal x
+            nonlocal x, y
 
-        exec 'global a; a = 1; b = 2' in g, l
-        if '__builtins__' in g: del g['__builtins__']
-        if '__builtins__' in l: del l['__builtins__']
-        if (g, l) != ({'a':1}, {'b':2}):
-            self.fail('exec ... in g (%s), l (%s)' %(g,l))
-
-    def test_assert(self):
+    def testAssert(self):
         # assertTruestmt: 'assert' test [',' test]
         assert 1
         assert 1, 1
@@ -741,7 +510,7 @@ hello world
     def testAssert2(self):
         try:
             assert 0, "msg"
-        except AssertionError, e:
+        except AssertionError as e:
             self.assertEqual(e.args[0], "msg")
         else:
             self.fail("AssertionError not raised by assert 0")
@@ -757,7 +526,7 @@ hello world
     ### compound_stmt: if_stmt | while_stmt | for_stmt | try_stmt | funcdef | classdef
     # Tested below
 
-    def test_if(self):
+    def testIf(self):
         # 'if' test ':' suite ('elif' test ':' suite)* ['else' ':' suite]
         if 1: pass
         if 1: pass
@@ -770,7 +539,7 @@ hello world
         elif 0: pass
         else: pass
 
-    def test_while(self):
+    def testWhile(self):
         # 'while' test ':' suite ['else' ':' suite]
         while 0: pass
         while 0: pass
@@ -785,7 +554,7 @@ hello world
             x = 2
         self.assertEqual(x, 2)
 
-    def test_for(self):
+    def testFor(self):
         # 'for' exprlist 'in' exprlist ':' suite ['else' ':' suite]
         for i in 1, 2, 3: pass
         for i, j, k in (): pass
@@ -812,30 +581,30 @@ hello world
             result.append(x)
         self.assertEqual(result, [1, 2, 3])
 
-    def test_try(self):
+    def testTry(self):
         ### try_stmt: 'try' ':' suite (except_clause ':' suite)+ ['else' ':' suite]
         ###         | 'try' ':' suite 'finally' ':' suite
-        ### except_clause: 'except' [expr [('as' | ',') expr]]
+        ### except_clause: 'except' [expr ['as' expr]]
         try:
-            1/0.0
+            1/0
         except ZeroDivisionError:
             pass
         else:
             pass
-        try: 1/0.0
+        try: 1/0
         except EOFError: pass
         except TypeError as msg: pass
-        except RuntimeError, msg: pass
+        except RuntimeError as msg: pass
         except: pass
         else: pass
-        try: 1/0.0
+        try: 1/0
         except (EOFError, TypeError, ZeroDivisionError): pass
-        try: 1/0.0
-        except (EOFError, TypeError, ZeroDivisionError), msg: pass
+        try: 1/0
+        except (EOFError, TypeError, ZeroDivisionError) as msg: pass
         try: pass
         finally: pass
 
-    def test_suite(self):
+    def testSuite(self):
         # simple_stmt | NEWLINE INDENT NEWLINE* (stmt NEWLINE*)+ DEDENT
         if 1: pass
         if 1:
@@ -850,7 +619,7 @@ hello world
             pass
             #
 
-    def test_test(self):
+    def testTest(self):
         ### and_test ('or' and_test)*
         ### and_test: not_test ('and' not_test)*
         ### not_test: 'not' not_test | comparison
@@ -861,9 +630,9 @@ hello world
         if not 1 and 1 and 1: pass
         if 1 and 1 or 1 and 1 and 1 or not 1 and 1: pass
 
-    def test_comparison(self):
+    def testComparison(self):
         ### comparison: expr (comp_op expr)*
-        ### comp_op: '<'|'>'|'=='|'>='|'<='|'<>'|'!='|'in'|'not' 'in'|'is'|'is' 'not'
+        ### comp_op: '<'|'>'|'=='|'>='|'<='|'!='|'in'|'not' 'in'|'is'|'is' 'not'
         if 1: pass
         x = (1 == 1)
         if 1 == 1: pass
@@ -877,56 +646,37 @@ hello world
         if 1 in (): pass
         if 1 not in (): pass
         if 1 < 1 > 1 == 1 >= 1 <= 1 != 1 in 1 not in 1 is 1 is not 1: pass
-        with check_py3k_warnings(('<> not supported in 3.x; use !=',
-                                  DeprecationWarning)):
-            if eval('1 <> 1'): pass
-        with check_py3k_warnings(('<> not supported in 3.x; use !=',
-                                  DeprecationWarning)):
-            if eval('1 < 1 > 1 == 1 >= 1 <= 1 <> 1 != 1 in 1 not in 1 is 1 is not 1'): pass
-        if sys.py3kwarning:
-            with warnings.catch_warnings():
-                warnings.filterwarnings('error', category=DeprecationWarning)
-                with self.assertRaises(DeprecationWarning) as cm:
-                    compile('1 <> 1', '<test string>', 'eval')
-                self.assertIn('<> not supported in 3.x; use !=',
-                              str(cm.exception))
 
-    def test_binary_mask_ops(self):
+    def testBinaryMaskOps(self):
         x = 1 & 1
         x = 1 ^ 1
         x = 1 | 1
 
-    def test_shift_ops(self):
+    def testShiftOps(self):
         x = 1 << 1
         x = 1 >> 1
         x = 1 << 1 >> 1
 
-    def test_additive_ops(self):
+    def testAdditiveOps(self):
         x = 1
         x = 1 + 1
         x = 1 - 1 - 1
         x = 1 - 1 + 1 - 1 + 1
 
-    def test_multiplicative_ops(self):
+    def testMultiplicativeOps(self):
         x = 1 * 1
-        with check_py3k_warnings(('classic int division', DeprecationWarning)):
-            x = 1 / 1
-        x = 1 / 1.0
+        x = 1 / 1
         x = 1 % 1
-        with check_py3k_warnings(('classic int division', DeprecationWarning)):
-            x = 1 / 1 * 1 % 1
-        x = 1 / 1.0 * 1 % 1
+        x = 1 / 1 * 1 % 1
 
-    def test_unary_ops(self):
+    def testUnaryOps(self):
         x = +1
         x = -1
         x = ~1
         x = ~1 ^ 1 & 1 | 1 & 1 ^ -1
-        with check_py3k_warnings(('classic int division', DeprecationWarning)):
-            x = -1*1/1 + 1*1 - ---1*1
-        x = -1*1/1.0 + 1*1 - ---1*1
+        x = -1*1/1 + 1*1 - ---1*1
 
-    def test_selectors(self):
+    def testSelectors(self):
         ### trailer: '(' [testlist] ')' | '[' subscript ']' | '.' NAME
         ### subscript: expr | [expr] ':' [expr]
 
@@ -953,12 +703,12 @@ hello world
         d[1,2] = 3
         d[1,2,3] = 4
         L = list(d)
-        L.sort(key=lambda x: (type(x).__name__, x))
+        L.sort(key=lambda x: x if isinstance(x, tuple) else ())
         self.assertEqual(str(L), '[1, (1,), (1, 2), (1, 2, 3)]')
 
-    def test_atoms(self):
-        ### atom: '(' [testlist] ')' | '[' [testlist] ']' | '{' [dictmaker] '}' | '`' testlist '`' | NAME | NUMBER | STRING
-        ### dictorsetmaker: (test ':' test (',' test ':' test)* [',']) | (test (',' test)* [','])
+    def testAtoms(self):
+        ### atom: '(' [testlist] ')' | '[' [testlist] ']' | '{' [dictsetmaker] '}' | NAME | NUMBER | STRING
+        ### dictsetmaker: (test ':' test (',' test ':' test)* [',']) | (test (',' test)* [','])
 
         x = (1)
         x = (1 or 2 or 3)
@@ -983,11 +733,6 @@ hello world
         x = {'one', 'two', 'three'}
         x = {2, 3, 4,}
 
-        with check_py3k_warnings(('backquote not supported', SyntaxWarning)):
-            x = eval('`x`')
-            x = eval('`1 or 2 or 3`')
-            self.assertEqual(eval('`1,2`'), '(1, 2)')
-
         x = x
         x = 'x'
         x = 123
@@ -996,7 +741,7 @@ hello world
     ### testlist: test (',' test)* [',']
     # These have been exercised enough above
 
-    def test_classdef(self):
+    def testClassdef(self):
         # 'class' NAME ['(' [testlist] ')'] ':' suite
         class B: pass
         class B2(): pass
@@ -1011,22 +756,18 @@ hello world
         # decorator: '@' dotted_name [ '(' [arglist] ')' ] NEWLINE
         # decorators: decorator+
         # decorated: decorators (classdef | funcdef)
-        def class_decorator(x):
-            x.decorated = True
-            return x
+        def class_decorator(x): return x
         @class_decorator
-        class G:
-            pass
-        self.assertEqual(G.decorated, True)
+        class G: pass
 
-    def test_dictcomps(self):
+    def testDictcomps(self):
         # dictorsetmaker: ( (test ':' test (comp_for |
         #                                   (',' test ':' test)* [','])) |
         #                   (test (comp_for | (',' test)* [','])) )
         nums = [1, 2, 3]
         self.assertEqual({i:i+1 for i in nums}, {1: 2, 2: 3, 3: 4})
 
-    def test_listcomps(self):
+    def testListcomps(self):
         # list comprehension tests
         nums = [1, 2, 3, 4, 5]
         strs = ["Apple", "Banana", "Coconut"]
@@ -1089,12 +830,12 @@ hello world
         self.assertEqual(x, [('Boeing', 'Airliner'), ('Boeing', 'Engine'), ('Ford', 'Engine'),
                              ('Macdonalds', 'Cheeseburger')])
 
-    def test_genexps(self):
+    def testGenexps(self):
         # generator expression tests
         g = ([x for x in range(10)] for x in range(1))
-        self.assertEqual(g.next(), [x for x in range(10)])
+        self.assertEqual(next(g), [x for x in range(10)])
         try:
-            g.next()
+            next(g)
             self.fail('should produce StopIteration exception')
         except StopIteration:
             pass
@@ -1102,7 +843,7 @@ hello world
         a = 1
         try:
             g = (a for d in a)
-            g.next()
+            next(g)
             self.fail('should produce TypeError')
         except TypeError:
             pass
@@ -1124,7 +865,7 @@ hello world
         check_syntax_error(self, "foo(x for x in range(10), 100)")
         check_syntax_error(self, "foo(100, x for x in range(10))")
 
-    def test_comprehension_specials(self):
+    def testComprehensionSpecials(self):
         # test for outmost iterable precomputation
         x = 10; g = (i for i in range(x)); x = 5
         self.assertEqual(len(list(g)), 10)
@@ -1163,14 +904,15 @@ hello world
         with manager() as x, manager():
             pass
 
-    def test_if_else_expr(self):
+    def testIfElseExpr(self):
         # Test ifelse expressions in various cases
         def _checkeval(msg, ret):
             "helper to check that evaluation of expressions is done correctly"
-            print(msg)
+            print(x)
             return ret
 
-        self.assertEqual([ x() for x in lambda: True, lambda: False if x() ], [True])
+        # the next line is not allowed anymore
+        #self.assertEqual([ x() for x in lambda: True, lambda: False if x() ], [True])
         self.assertEqual([ x() for x in (lambda: True, lambda: False) if x() ], [True])
         self.assertEqual([ x(False) for x in (lambda x: False if x else True, lambda x: True if x else False) if x(False) ], [True])
         self.assertEqual((5 if 1 else _checkeval("check 1", 0)), 5)
@@ -1186,8 +928,7 @@ hello world
         self.assertEqual((6 + 1 if 1 else 2), 7)
         self.assertEqual((6 - 1 if 1 else 2), 5)
         self.assertEqual((6 * 2 if 1 else 4), 12)
-        with check_py3k_warnings(('classic int division', DeprecationWarning)):
-            self.assertEqual((6 / 2 if 1 else 3), 3)
+        self.assertEqual((6 / 2 if 1 else 3), 3)
         self.assertEqual((6 < 4 if 0 else 2), 2)
 
     def test_paren_evaluation(self):
