@@ -1,35 +1,46 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 """Test script for the gzip module.
 """
 
 import unittest
-from test import test_support
+from test import support
 import os
 import io
 import struct
-gzip = test_support.import_module('gzip')
+gzip = support.import_module('gzip')
 
-data1 = """  int length=DEFAULTALLOC, err = Z_OK;
+data1 = b"""  int length=DEFAULTALLOC, err = Z_OK;
   PyObject *RetVal;
   int flushmode = Z_FINISH;
   unsigned long start_total_out;
 
 """
 
-data2 = """/* zlibmodule.c -- gzip-compatible data compression */
+data2 = b"""/* zlibmodule.c -- gzip-compatible data compression */
 /* See http://www.gzip.org/zlib/
 /* See http://www.winimage.com/zLibDll for Windows */
 """
 
 
+class UnseekableIO(io.BytesIO):
+    def seekable(self):
+        return False
+
+    def tell(self):
+        raise io.UnsupportedOperation
+
+    def seek(self, *args):
+        raise io.UnsupportedOperation
+
+
 class TestGzip(unittest.TestCase):
-    filename = test_support.TESTFN
+    filename = support.TESTFN
 
     def setUp(self):
-        test_support.unlink(self.filename)
+        support.unlink(self.filename)
 
     def tearDown(self):
-        test_support.unlink(self.filename)
+        support.unlink(self.filename)
 
 
     def test_write(self):
@@ -53,13 +64,6 @@ class TestGzip(unittest.TestCase):
             d = f.read()
         self.assertEqual(d, data1*50)
 
-    def test_read_universal_newlines(self):
-        # Issue #5148: Reading breaks when mode contains 'U'.
-        self.test_write()
-        with gzip.GzipFile(self.filename, 'rU') as f:
-            d = f.read()
-        self.assertEqual(d, data1*50)
-
     def test_io_on_closed_object(self):
         # Test that I/O operations on closed GzipFile objects raise a
         # ValueError, just like the corresponding functions on file objects.
@@ -78,7 +82,7 @@ class TestGzip(unittest.TestCase):
         f = gzip.GzipFile(self.filename, 'w')
         f.close()
         with self.assertRaises(ValueError):
-            f.write('')
+            f.write(b'')
         with self.assertRaises(ValueError):
             f.flush()
 
@@ -97,19 +101,19 @@ class TestGzip(unittest.TestCase):
         # many, many members.  Create such a file and verify that reading it
         # works.
         with gzip.open(self.filename, 'wb', 9) as f:
-            f.write('a')
+            f.write(b'a')
         for i in range(0, 200):
             with gzip.open(self.filename, "ab", 9) as f: # append
-                f.write('a')
+                f.write(b'a')
 
         # Try reading the file
         with gzip.open(self.filename, "rb") as zgfile:
-            contents = ""
+            contents = b""
             while 1:
                 ztxt = zgfile.read(8192)
                 contents += ztxt
                 if not ztxt: break
-        self.assertEqual(contents, 'a'*201)
+        self.assertEqual(contents, b'a'*201)
 
     def test_buffered_reader(self):
         # Issue #7471: a GzipFile can be wrapped in a BufferedReader for
@@ -180,7 +184,7 @@ class TestGzip(unittest.TestCase):
         with gzip.GzipFile(self.filename, 'w') as f:
             for pos in range(0, 256, 16):
                 f.seek(pos)
-                f.write('GZ\n')
+                f.write(b'GZ\n')
 
     def test_mode(self):
         self.test_write()
@@ -193,6 +197,12 @@ class TestGzip(unittest.TestCase):
                 self.assertTrue(hasattr(f, "name"))
                 self.assertEqual(f.name, self.filename)
 
+    def test_paddedfile_getattr(self):
+        self.test_write()
+        with gzip.GzipFile(self.filename, 'rb') as f:
+            self.assertTrue(hasattr(f.fileobj, "name"))
+            self.assertEqual(f.fileobj.name, self.filename)
+
     def test_mtime(self):
         mtime = 123456789
         with gzip.GzipFile(self.filename, 'w', mtime = mtime) as fWrite:
@@ -213,28 +223,28 @@ class TestGzip(unittest.TestCase):
             # see RFC 1952: http://www.faqs.org/rfcs/rfc1952.html
 
             idBytes = fRead.read(2)
-            self.assertEqual(idBytes, '\x1f\x8b') # gzip ID
+            self.assertEqual(idBytes, b'\x1f\x8b') # gzip ID
 
             cmByte = fRead.read(1)
-            self.assertEqual(cmByte, '\x08') # deflate
+            self.assertEqual(cmByte, b'\x08') # deflate
 
             flagsByte = fRead.read(1)
-            self.assertEqual(flagsByte, '\x08') # only the FNAME flag is set
+            self.assertEqual(flagsByte, b'\x08') # only the FNAME flag is set
 
             mtimeBytes = fRead.read(4)
             self.assertEqual(mtimeBytes, struct.pack('<i', mtime)) # little-endian
 
             xflByte = fRead.read(1)
-            self.assertEqual(xflByte, '\x02') # maximum compression
+            self.assertEqual(xflByte, b'\x02') # maximum compression
 
             osByte = fRead.read(1)
-            self.assertEqual(osByte, '\xff') # OS "unknown" (OS-independent)
+            self.assertEqual(osByte, b'\xff') # OS "unknown" (OS-independent)
 
             # Since the FNAME flag is set, the zero-terminated filename follows.
             # RFC 1952 specifies that this is the name of the input file, if any.
             # However, the gzip module defaults to storing the name of the output
             # file in this field.
-            expected = self.filename.encode('Latin-1') + '\x00'
+            expected = self.filename.encode('Latin-1') + b'\x00'
             nameBytes = fRead.read(len(expected))
             self.assertEqual(nameBytes, expected)
 
@@ -243,7 +253,7 @@ class TestGzip(unittest.TestCase):
             fRead.seek(os.stat(self.filename).st_size - 8)
 
             crc32Bytes = fRead.read(4) # CRC32 of uncompressed data [data1]
-            self.assertEqual(crc32Bytes, '\xaf\xd7d\x83')
+            self.assertEqual(crc32Bytes, b'\xaf\xd7d\x83')
 
             isizeBytes = fRead.read(4)
             self.assertEqual(isizeBytes, struct.pack('<i', len(data1)))
@@ -263,11 +273,11 @@ class TestGzip(unittest.TestCase):
             self.fail("__enter__ on a closed file didn't raise an exception")
         try:
             with gzip.GzipFile(self.filename, "wb") as f:
-                1 // 0
+                1/0
         except ZeroDivisionError:
             pass
         else:
-            self.fail("1 // 0 didn't raise an exception")
+            self.fail("1/0 didn't raise an exception")
 
     def test_zero_padded_file(self):
         with gzip.GzipFile(self.filename, "wb") as f:
@@ -275,30 +285,74 @@ class TestGzip(unittest.TestCase):
 
         # Pad the file with zeroes
         with open(self.filename, "ab") as f:
-            f.write("\x00" * 50)
+            f.write(b"\x00" * 50)
 
         with gzip.GzipFile(self.filename, "rb") as f:
             d = f.read()
             self.assertEqual(d, data1 * 50, "Incorrect data in file")
 
+    def test_non_seekable_file(self):
+        uncompressed = data1 * 50
+        buf = UnseekableIO()
+        with gzip.GzipFile(fileobj=buf, mode="wb") as f:
+            f.write(uncompressed)
+        compressed = buf.getvalue()
+        buf = UnseekableIO(compressed)
+        with gzip.GzipFile(fileobj=buf, mode="rb") as f:
+            self.assertEqual(f.read(), uncompressed)
+
+    def test_peek(self):
+        uncompressed = data1 * 200
+        with gzip.GzipFile(self.filename, "wb") as f:
+            f.write(uncompressed)
+
+        def sizes():
+            while True:
+                for n in range(5, 50, 10):
+                    yield n
+
+        with gzip.GzipFile(self.filename, "rb") as f:
+            f.max_read_chunk = 33
+            nread = 0
+            for n in sizes():
+                s = f.peek(n)
+                if s == b'':
+                    break
+                self.assertEqual(f.read(len(s)), s)
+                nread += len(s)
+            self.assertEqual(f.read(100), b'')
+            self.assertEqual(nread, len(uncompressed))
+
     def test_fileobj_from_fdopen(self):
-        # Issue #13781: Creating a GzipFile using a fileobj from os.fdopen()
-        # should not embed the fake filename "<fdopen>" in the output file.
+        # Issue #13781: Opening a GzipFile for writing fails when using a
+        # fileobj created with os.fdopen().
         fd = os.open(self.filename, os.O_WRONLY | os.O_CREAT)
         with os.fdopen(fd, "wb") as f:
             with gzip.GzipFile(fileobj=f, mode="w") as g:
-                self.assertEqual(g.name, "")
+                pass
 
-    def test_read_with_extra(self):
-        # Gzip data with an extra field
-        gzdata = (b'\x1f\x8b\x08\x04\xb2\x17cQ\x02\xff'
-                  b'\x05\x00Extra'
-                  b'\x0bI-.\x01\x002\xd1Mx\x04\x00\x00\x00')
-        with gzip.GzipFile(fileobj=io.BytesIO(gzdata)) as f:
-            self.assertEqual(f.read(), b'Test')
+    # Testing compress/decompress shortcut functions
+
+    def test_compress(self):
+        for data in [data1, data2]:
+            for args in [(), (1,), (6,), (9,)]:
+                datac = gzip.compress(data, *args)
+                self.assertEqual(type(datac), bytes)
+                with gzip.GzipFile(fileobj=io.BytesIO(datac), mode="rb") as f:
+                    self.assertEqual(f.read(), data)
+
+    def test_decompress(self):
+        for data in (data1, data2):
+            buf = io.BytesIO()
+            with gzip.GzipFile(fileobj=buf, mode="wb") as f:
+                f.write(data)
+            self.assertEqual(gzip.decompress(buf.getvalue()), data)
+            # Roundtrip with compress
+            datac = gzip.compress(data)
+            self.assertEqual(gzip.decompress(datac), data)
 
 def test_main(verbose=None):
-    test_support.run_unittest(TestGzip)
+    support.run_unittest(TestGzip)
 
 if __name__ == "__main__":
     test_main(verbose=True)
