@@ -1,11 +1,18 @@
+#!/usr/bin/env python3
+
 import unittest
-from test import test_support
+from test import support
 from test.test_urllib2 import sanepathname2url
 
-import socket
-import urllib2
 import os
+import socket
+import urllib.error
+import urllib.request
 import sys
+try:
+    import ssl
+except ImportError:
+    ssl = None
 
 TIMEOUT = 60  # seconds
 
@@ -14,7 +21,8 @@ def _retry_thrice(func, exc, *args, **kwargs):
     for i in range(3):
         try:
             return func(*args, **kwargs)
-        except exc, last_exc:
+        except exc as e:
+            last_exc = e
             continue
         except:
             raise
@@ -25,16 +33,10 @@ def _wrap_with_retry_thrice(func, exc):
         return _retry_thrice(func, exc, *args, **kwargs)
     return wrapped
 
-# bpo-35411: FTP tests of test_urllib2net randomly fail
-# with "425 Security: Bad IP connecting" on Travis CI
-skip_ftp_test_on_travis = unittest.skipIf('TRAVIS' in os.environ,
-                                          'bpo-35411: skip FTP test '
-                                          'on Travis CI')
-
-
 # Connecting to remote hosts is flaky.  Make it more robust by retrying
 # the connection several times.
-_urlopen_with_retry = _wrap_with_retry_thrice(urllib2.urlopen, urllib2.URLError)
+_urlopen_with_retry = _wrap_with_retry_thrice(urllib.request.urlopen,
+                                              urllib.error.URLError)
 
 
 class AuthTests(unittest.TestCase):
@@ -44,7 +46,7 @@ class AuthTests(unittest.TestCase):
 ## could be used to HTTP authentication.
 #
 #    def test_basic_auth(self):
-#        import httplib
+#        import http.client
 #
 #        test_url = "http://www.python.org/test/test_urllib2/basic_auth"
 #        test_hostport = "www.python.org"
@@ -72,30 +74,21 @@ class AuthTests(unittest.TestCase):
 #        # reasons, let's not implement it!  (it's already implemented for proxy
 #        # specification strings (that is, URLs or authorities specifying a
 #        # proxy), so we must keep that)
-#        self.assertRaises(httplib.InvalidURL,
+#        self.assertRaises(http.client.InvalidURL,
 #                          urllib2.urlopen, "http://evil:thing@example.com")
 
 
 class CloseSocketTest(unittest.TestCase):
 
     def test_close(self):
-        import httplib
-
         # calling .close() on urllib2's response objects should close the
         # underlying socket
 
-        # delve deep into response to fetch socket._socketobject
-        response = _urlopen_with_retry(test_support.TEST_HTTP_URL)
-        abused_fileobject = response.fp
-        self.assertIs(abused_fileobject.__class__, socket._fileobject)
-        httpresponse = abused_fileobject._sock
-        self.assertIs(httpresponse.__class__, httplib.HTTPResponse)
-        fileobject = httpresponse.fp
-        self.assertIs(fileobject.__class__, socket._fileobject)
-
-        self.assertTrue(not fileobject.closed)
+        response = _urlopen_with_retry("http://www.python.org/")
+        sock = response.fp
+        self.assertTrue(not sock.closed)
         response.close()
-        self.assertTrue(fileobject.closed)
+        self.assertTrue(sock.closed)
 
 class OtherNetworkTests(unittest.TestCase):
     def setUp(self):
@@ -107,30 +100,32 @@ class OtherNetworkTests(unittest.TestCase):
     # XXX The rest of these tests aren't very good -- they don't check much.
     # They do sometimes catch some major disasters, though.
 
-    @skip_ftp_test_on_travis
     def test_ftp(self):
         urls = [
-            'ftp://www.pythontest.net/README',
-            ('ftp://www.pythontest.net/non-existent-file',
-             None, urllib2.URLError),
+            'ftp://ftp.kernel.org/pub/linux/kernel/README',
+            'ftp://ftp.kernel.org/pub/linux/kernel/non-existent-file',
+            #'ftp://ftp.kernel.org/pub/leenox/kernel/test',
+            'ftp://gatekeeper.research.compaq.com/pub/DEC/SRC'
+                '/research-reports/00README-Legal-Rules-Regs',
             ]
         self._test_urls(urls, self._extra_handlers())
 
     def test_file(self):
-        TESTFN = test_support.TESTFN
+        TESTFN = support.TESTFN
         f = open(TESTFN, 'w')
         try:
             f.write('hi there\n')
             f.close()
             urls = [
-                'file:'+sanepathname2url(os.path.abspath(TESTFN)),
-                ('file:///nonsensename/etc/passwd', None, urllib2.URLError),
+                'file:' + sanepathname2url(os.path.abspath(TESTFN)),
+                ('file:///nonsensename/etc/passwd', None,
+                 urllib.error.URLError),
                 ]
             self._test_urls(urls, self._extra_handlers(), retry=True)
         finally:
             os.remove(TESTFN)
 
-        self.assertRaises(ValueError, urllib2.urlopen,'./relative_path/to/file')
+        self.assertRaises(ValueError, urllib.request.urlopen,'./relative_path/to/file')
 
     # XXX Following test depends on machine configurations that are internal
     # to CNRI.  Need to set up a public server with the right authentication
@@ -161,29 +156,18 @@ class OtherNetworkTests(unittest.TestCase):
 ##             self._test_urls(urls, self._extra_handlers()+[bauth, dauth])
 
     def test_urlwithfrag(self):
-        urlwith_frag = "http://www.pythontest.net/index.html#frag"
-        with test_support.transient_internet(urlwith_frag):
-            req = urllib2.Request(urlwith_frag)
-            res = urllib2.urlopen(req)
+        urlwith_frag = "http://docs.python.org/glossary.html#glossary"
+        with support.transient_internet(urlwith_frag):
+            req = urllib.request.Request(urlwith_frag)
+            res = urllib.request.urlopen(req)
             self.assertEqual(res.geturl(),
-                    "http://www.pythontest.net/index.html#frag")
-
-    def test_fileno(self):
-        req = urllib2.Request(test_support.TEST_HTTP_URL)
-        opener = urllib2.build_opener()
-        res = opener.open(req)
-        try:
-            res.fileno()
-        except AttributeError:
-            self.fail("HTTPResponse object should return a valid fileno")
-        finally:
-            res.close()
+                    "http://docs.python.org/glossary.html#glossary")
 
     def test_custom_headers(self):
-        url = test_support.TEST_HTTP_URL
-        with test_support.transient_internet(url):
-            opener = urllib2.build_opener()
-            request = urllib2.Request(url)
+        url = "http://www.example.com"
+        with support.transient_internet(url):
+            opener = urllib.request.build_opener()
+            request = urllib.request.Request(url)
             self.assertFalse(request.header_items())
             opener.open(request)
             self.assertTrue(request.header_items())
@@ -192,14 +176,23 @@ class OtherNetworkTests(unittest.TestCase):
             opener.open(request)
             self.assertEqual(request.get_header('User-agent'),'Test-Agent')
 
-    @unittest.skip('XXX: http://www.imdb.com is gone')
     def test_sites_no_connection_close(self):
         # Some sites do not send Connection: close header.
         # Verify that those work properly. (#issue12576)
 
-        URL = 'http://www.imdb.com' # No Connection:close
-        with test_support.transient_internet(URL):
-            req = urllib2.urlopen(URL)
+        URL = 'http://www.imdb.com' # mangles Connection:close
+
+        with support.transient_internet(URL):
+            try:
+                with urllib.request.urlopen(URL) as res:
+                    pass
+            except ValueError as e:
+                self.fail("urlopen failed for site not sending \
+                           Connection:close")
+            else:
+                self.assertTrue(res)
+
+            req = urllib.request.urlopen(URL)
             res = req.read()
             self.assertTrue(res)
 
@@ -208,16 +201,17 @@ class OtherNetworkTests(unittest.TestCase):
         import logging
         debug = logging.getLogger("test_urllib2").debug
 
-        urlopen = urllib2.build_opener(*handlers).open
+        urlopen = urllib.request.build_opener(*handlers).open
         if retry:
-            urlopen = _wrap_with_retry_thrice(urlopen, urllib2.URLError)
+            urlopen = _wrap_with_retry_thrice(urlopen, urllib.error.URLError)
 
         for url in urls:
             if isinstance(url, tuple):
                 url, req, expected_err = url
             else:
                 req = expected_err = None
-            with test_support.transient_internet(url):
+
+            with support.transient_internet(url):
                 debug(url)
                 try:
                     f = urlopen(url, req, TIMEOUT)
@@ -227,19 +221,21 @@ class OtherNetworkTests(unittest.TestCase):
                         msg = ("Didn't get expected error(s) %s for %s %s, got %s: %s" %
                                (expected_err, url, req, type(err), err))
                         self.assertIsInstance(err, expected_err, msg)
-                except urllib2.URLError as err:
+                except urllib.error.URLError as err:
                     if isinstance(err[0], socket.timeout):
-                        print >>sys.stderr, "<timeout: %s>" % url
+                        print("<timeout: %s>" % url, file=sys.stderr)
                         continue
                     else:
                         raise
                 else:
                     try:
-                        with test_support.transient_internet(url):
+                        with support.time_out, \
+                             support.socket_peer_reset, \
+                             support.ioerror_peer_reset:
                             buf = f.read()
                             debug("read %d bytes" % len(buf))
                     except socket.timeout:
-                        print >>sys.stderr, "<timeout: %s>" % url
+                        print("<timeout: %s>" % url, file=sys.stderr)
                     f.close()
             debug("******** next url coming up...")
             time.sleep(0.1)
@@ -247,7 +243,7 @@ class OtherNetworkTests(unittest.TestCase):
     def _extra_handlers(self):
         handlers = []
 
-        cfh = urllib2.CacheFTPHandler()
+        cfh = urllib.request.CacheFTPHandler()
         self.addCleanup(cfh.clear_cache)
         cfh.setTimeout(1)
         handlers.append(cfh)
@@ -257,85 +253,111 @@ class OtherNetworkTests(unittest.TestCase):
 
 class TimeoutTest(unittest.TestCase):
     def test_http_basic(self):
-        self.assertIsNone(socket.getdefaulttimeout())
-        url = test_support.TEST_HTTP_URL
-        with test_support.transient_internet(url, timeout=None):
+        self.assertTrue(socket.getdefaulttimeout() is None)
+        url = "http://www.python.org"
+        with support.transient_internet(url, timeout=None):
             u = _urlopen_with_retry(url)
-            self.assertIsNone(u.fp._sock.fp._sock.gettimeout())
+            self.addCleanup(u.close)
+            self.assertTrue(u.fp.raw._sock.gettimeout() is None)
 
     def test_http_default_timeout(self):
-        self.assertIsNone(socket.getdefaulttimeout())
-        url = test_support.TEST_HTTP_URL
-        with test_support.transient_internet(url):
+        self.assertTrue(socket.getdefaulttimeout() is None)
+        url = "http://www.python.org"
+        with support.transient_internet(url):
             socket.setdefaulttimeout(60)
             try:
                 u = _urlopen_with_retry(url)
+                self.addCleanup(u.close)
             finally:
                 socket.setdefaulttimeout(None)
-            self.assertEqual(u.fp._sock.fp._sock.gettimeout(), 60)
+            self.assertEqual(u.fp.raw._sock.gettimeout(), 60)
 
     def test_http_no_timeout(self):
-        self.assertIsNone(socket.getdefaulttimeout())
-        url = test_support.TEST_HTTP_URL
-        with test_support.transient_internet(url):
+        self.assertTrue(socket.getdefaulttimeout() is None)
+        url = "http://www.python.org"
+        with support.transient_internet(url):
             socket.setdefaulttimeout(60)
             try:
                 u = _urlopen_with_retry(url, timeout=None)
+                self.addCleanup(u.close)
             finally:
                 socket.setdefaulttimeout(None)
-            self.assertIsNone(u.fp._sock.fp._sock.gettimeout())
+            self.assertTrue(u.fp.raw._sock.gettimeout() is None)
 
     def test_http_timeout(self):
-        url = test_support.TEST_HTTP_URL
-        with test_support.transient_internet(url):
+        url = "http://www.python.org"
+        with support.transient_internet(url):
             u = _urlopen_with_retry(url, timeout=120)
-            self.assertEqual(u.fp._sock.fp._sock.gettimeout(), 120)
+            self.addCleanup(u.close)
+            self.assertEqual(u.fp.raw._sock.gettimeout(), 120)
 
-    FTP_HOST = 'ftp://www.pythontest.net/'
+    FTP_HOST = "ftp://ftp.mirror.nl/pub/gnu/"
 
-    @skip_ftp_test_on_travis
     def test_ftp_basic(self):
-        self.assertIsNone(socket.getdefaulttimeout())
-        with test_support.transient_internet(self.FTP_HOST, timeout=None):
+        self.assertTrue(socket.getdefaulttimeout() is None)
+        with support.transient_internet(self.FTP_HOST, timeout=None):
             u = _urlopen_with_retry(self.FTP_HOST)
-            self.assertIsNone(u.fp.fp._sock.gettimeout())
+            self.addCleanup(u.close)
+            self.assertTrue(u.fp.fp.raw._sock.gettimeout() is None)
 
-    @skip_ftp_test_on_travis
     def test_ftp_default_timeout(self):
-        self.assertIsNone(socket.getdefaulttimeout())
-        with test_support.transient_internet(self.FTP_HOST):
+        self.assertTrue(socket.getdefaulttimeout() is None)
+        with support.transient_internet(self.FTP_HOST):
             socket.setdefaulttimeout(60)
             try:
                 u = _urlopen_with_retry(self.FTP_HOST)
+                self.addCleanup(u.close)
             finally:
                 socket.setdefaulttimeout(None)
-            self.assertEqual(u.fp.fp._sock.gettimeout(), 60)
+            self.assertEqual(u.fp.fp.raw._sock.gettimeout(), 60)
 
-    @skip_ftp_test_on_travis
     def test_ftp_no_timeout(self):
-        self.assertIsNone(socket.getdefaulttimeout(),)
-        with test_support.transient_internet(self.FTP_HOST):
+        self.assertTrue(socket.getdefaulttimeout() is None)
+        with support.transient_internet(self.FTP_HOST):
             socket.setdefaulttimeout(60)
             try:
                 u = _urlopen_with_retry(self.FTP_HOST, timeout=None)
+                self.addCleanup(u.close)
             finally:
                 socket.setdefaulttimeout(None)
-            self.assertIsNone(u.fp.fp._sock.gettimeout())
+            self.assertTrue(u.fp.fp.raw._sock.gettimeout() is None)
 
-    @skip_ftp_test_on_travis
     def test_ftp_timeout(self):
-        with test_support.transient_internet(self.FTP_HOST):
+        with support.transient_internet(self.FTP_HOST):
             u = _urlopen_with_retry(self.FTP_HOST, timeout=60)
-            self.assertEqual(u.fp.fp._sock.gettimeout(), 60)
+            self.addCleanup(u.close)
+            self.assertEqual(u.fp.fp.raw._sock.gettimeout(), 60)
+
+
+@unittest.skipUnless(ssl, "requires SSL support")
+class HTTPSTests(unittest.TestCase):
+
+    def test_sni(self):
+        self.skipTest("test disabled - test server needed")
+        # Checks that Server Name Indication works, if supported by the
+        # OpenSSL linked to.
+        # The ssl module itself doesn't have server-side support for SNI,
+        # so we rely on a third-party test site.
+        expect_sni = ssl.HAS_SNI
+        with support.transient_internet("XXX"):
+            u = urllib.request.urlopen("XXX")
+            contents = u.readall()
+            if expect_sni:
+                self.assertIn(b"Great", contents)
+                self.assertNotIn(b"Unfortunately", contents)
+            else:
+                self.assertNotIn(b"Great", contents)
+                self.assertIn(b"Unfortunately", contents)
 
 
 def test_main():
-    test_support.requires("network")
-    test_support.run_unittest(AuthTests,
-                              OtherNetworkTests,
-                              CloseSocketTest,
-                              TimeoutTest,
-                              )
+    support.requires("network")
+    support.run_unittest(AuthTests,
+                         HTTPSTests,
+                         OtherNetworkTests,
+                         CloseSocketTest,
+                         TimeoutTest,
+                         )
 
 if __name__ == "__main__":
     test_main()
